/***************************************************************************
 *   Copyright (C) 2009 by Saïd LANKRI   *
 *   said.lankri@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ThemeBuilderUI.h"

#include <QComboBox>
#include <QSettings>
#include <QFile>
#include <QListWidget>
#include <QFileDialog>

#include "../themeconfig/ThemeConfig.h"

ThemeBuilderUI::ThemeBuilderUI(QWidget* parent)
 : QWidget(parent), config(NULL)
{
  setupUi(this);

  config = new ThemeConfig();
  defaultConfig = new ThemeConfig(":default.qsconfig");

  QObject::connect(widgetList,SIGNAL(currentTextChanged(const QString &)), this,SLOT(slot_WidgetChanged(const QString &)));
  QObject::connect(quitButton,SIGNAL(clicked()), this,SLOT(close()));
  QObject::connect(loadButton,SIGNAL(clicked()), this,SLOT(slot_open()));
  QObject::connect(newButton,SIGNAL(clicked()), this,SLOT(slot_new()));
}

ThemeBuilderUI::~ThemeBuilderUI()
{
  delete config;
  delete defaultConfig;
}

void ThemeBuilderUI::slot_quit()
{
  slot_save(lastWidget);
}

void ThemeBuilderUI::slot_loadTheme(const QString& theme)
{
  slot_save(lastWidget);
  widgetList->clear();
  svgElements.clear();
  config->load(theme);

  if (config->isValid()) {
    QStringList f = theme.split("/");
    if (f.size() > 0)
      themeLabel->setText(f[f.size()-1]);

    QStringList l = config->getWidgets();
    QStringList::iterator it;
    widgetList->addItems(l);

    for (it=l.begin(); it!=l.end(); ++it) {
      widget_spec_t wspec = config->getWidgetSpec(*it);
      if (!wspec.frame.element.isNull())
        if (!svgElements.contains(wspec.frame.element))
          svgElements.insert(0,wspec.frame.element);

      if (!wspec.interior.element.isNull())
        if (!svgElements.contains(wspec.interior.element))
          svgElements.insert(0,wspec.interior.element);

      if (!wspec.indicator.element.isNull())
        if (!svgElements.contains(wspec.indicator.element))
          svgElements.insert(0,wspec.indicator.element);
    }

    // add missing widgets from default config
    // TODO
  }
}

void ThemeBuilderUI::slot_WidgetChanged(const QString& widget)
{
  if (!config->isValid())
    return;

  if (!lastWidget.isEmpty())
    slot_save(lastWidget);

  lastWidget = QString();

  if (widget.isEmpty())
    return;

  lastWidget = widget;

  widget_spec_t wspec = config->getWidgetSpec(widget);

  printf("load %s\n",(const char *)(widget.toAscii()));

  capsuleCb->setChecked(wspec.frame.hasCapsule);

  // frame
  frameSvgCombo->clear();
  frameSvgCombo->addItems(svgElements);
  frameCb->setChecked(wspec.frame.hasFrame);
  frameSvgCombo->setCurrentIndex(frameSvgCombo->findText(wspec.frame.element));
  frameTopSb->setValue(wspec.frame.top);
  frameBottomSb->setValue(wspec.frame.bottom);
  frameLeftSb->setValue(wspec.frame.left);
  frameRightSb->setValue(wspec.frame.right);
  frameTopRxCb->setChecked(wspec.frame.rtop);
  frameBottomRxCb->setChecked(wspec.frame.rbottom);
  frameLeftRxCb->setChecked(wspec.frame.rleft);
  frameRightRxCb->setChecked(wspec.frame.rright);
  frameTopRxSb->setValue(wspec.frame.ptop);
  frameBottomRxSb->setValue(wspec.frame.pbottom);
  frameLeftRxSb->setValue(wspec.frame.pleft);
  frameRightRxSb->setValue(wspec.frame.pright);

  // interior
  interiorSvgCombo->clear();
  interiorSvgCombo->addItems(svgElements);
  interiorCb->setChecked(wspec.interior.hasInterior);
  interiorSvgCombo->setCurrentIndex(interiorSvgCombo->findText(wspec.interior.element));
  interiorRxCb->setChecked(wspec.interior.rx);
  interiorRyCb->setChecked(wspec.interior.ry);
  interiorRxSb->setValue(wspec.interior.px);
  interiorRySb->setValue(wspec.interior.py);
  interiorMarginCb->setChecked(wspec.interior.hasMargin);
  interiorTopSb->setValue(wspec.interior.top);
  interiorBottomSb->setValue(wspec.interior.bottom);
  interiorLeftSb->setValue(wspec.interior.left);
  interiorRightSb->setValue(wspec.interior.right);

  // indicator
  indicatorSvgCombo->clear();
  indicatorSvgCombo->addItems(svgElements);
  indicatorSvgCombo->setCurrentIndex(indicatorSvgCombo->findText(wspec.indicator.element));
  indicatorSizeSb->setValue(wspec.indicator.size);

  // text
  shadowCb->setChecked(wspec.label.hasShadow);
  shadowXSb->setValue(wspec.label.xshift);
  shadowYSb->setValue(wspec.label.yshift);
  shadowDepthSb->setValue(wspec.label.depth);
  shadowRedSb->setValue(wspec.label.r);
  shadowGreenSb->setValue(wspec.label.g);
  shadowBlueSb->setValue(wspec.label.b);
  shadowAlphaSb->setValue(wspec.label.a);
  textMarginCb->setChecked(wspec.label.hasMargin);
  textTopSb->setValue(wspec.label.top);
  textBottomSb->setValue(wspec.label.bottom);
  textLeftSb->setValue(wspec.label.left);
  textRightSb->setValue(wspec.label.right);
  textIconSpacingSb->setValue(wspec.label.tispace);

  // size
  heightSb->setValue(0);
  widthSb->setValue(0);

  if (wspec.size.minH > 0) {
    heightSb->setValue(wspec.size.minH);
    minHeightRadio->setChecked(true);
    forceHeightCb->setChecked(true);
  }
  if (wspec.size.fixedH > 0) {
    heightSb->setValue(wspec.size.fixedH);
    fixedHeightRadio->setChecked(true);
    forceHeightCb->setChecked(true);
  }
  if (wspec.size.minW > 0) {
    widthSb->setValue(wspec.size.minW);
    minWidthRadio->setChecked(true);
    forceWidthCb->setChecked(true);
  }
  if (wspec.size.fixedW > 0) {
    widthSb->setValue(wspec.size.fixedW);
    fixedWidthRadio->setChecked(true);
    forceWidthCb->setChecked(true);
  }
  if ( (wspec.size.minH <= 0) && (wspec.size.fixedH <= 0) )
    forceHeightCb->setChecked(false);
  if ( (wspec.size.minW <= 0) && (wspec.size.fixedW <= 0) )
    forceWidthCb->setChecked(false);
}

void ThemeBuilderUI::slot_save(const QString& widget)
{
  if (!config->isValid())
    return;

  if (widget.isEmpty())
    return;

  widget_spec_t wspec;
  wspec.frame.hasFrame = frameCb->isChecked();
  wspec.frame.hasCapsule = capsuleCb->isChecked();
  wspec.frame.element = frameSvgCombo->currentText();
  if (!svgElements.contains(wspec.frame.element))
    svgElements.append(wspec.frame.element);
  wspec.frame.top = frameTopSb->value();
  wspec.frame.bottom = frameBottomSb->value();
  wspec.frame.left = frameLeftSb->value();
  wspec.frame.right = frameRightSb->value();
  wspec.frame.rtop = frameTopRxCb->isChecked();
  wspec.frame.rbottom = frameBottomRxCb->isChecked();
  wspec.frame.rleft = frameLeftRxCb->isChecked();
  wspec.frame.rright = frameRightRxCb->isChecked();
  wspec.frame.ptop = frameTopSb->value();
  wspec.frame.pbottom = frameBottomSb->value();
  wspec.frame.pleft = frameLeftSb->value();
  wspec.frame.pright = frameRightSb->value();

  wspec.interior.hasInterior = interiorCb->isChecked();
  wspec.interior.element = interiorSvgCombo->currentText();
  if (!svgElements.contains(wspec.interior.element))
    svgElements.append(wspec.interior.element);
  wspec.interior.hasMargin = interiorMarginCb->isChecked();
  wspec.interior.top = interiorTopSb->value();
  wspec.interior.bottom = interiorBottomSb->value();
  wspec.interior.left = interiorLeftSb->value();
  wspec.interior.right = interiorRightSb->value();
  wspec.interior.rx = interiorRxCb->isChecked();
  wspec.interior.ry = interiorRyCb->isChecked();
  wspec.interior.px = interiorRxSb->value();
  wspec.interior.py = interiorRySb->value();

  wspec.indicator.element = indicatorSvgCombo->currentText();
  if (!svgElements.contains(wspec.indicator.element))
    svgElements.append(wspec.indicator.element);
  wspec.indicator.size = indicatorSizeSb->value();

  wspec.label.hasShadow = shadowCb->isChecked();
  wspec.label.xshift = shadowXSb->value();
  wspec.label.yshift = shadowYSb->value();
  wspec.label.r = shadowRedSb->value();
  wspec.label.g = shadowGreenSb->value();
  wspec.label.b = shadowBlueSb->value();
  wspec.label.a = shadowAlphaSb->value();
  wspec.label.depth = shadowDepthSb->value();
  wspec.label.hasMargin = textMarginCb->isChecked();
  wspec.label.top = textTopSb->value();
  wspec.label.bottom = textBottomSb->value();
  wspec.label.left = textLeftSb->value();
  wspec.label.right = textRightSb->value();
  wspec.label.tispace = textIconSpacingSb->value();

  wspec.size.minH = wspec.size.fixedH = 0;
  wspec.size.minW = wspec.size.fixedW = 0;

  if (forceHeightCb->isChecked()) {
    if (minHeightRadio->isChecked()) {
      wspec.size.minH = heightSb->value();
      wspec.size.fixedH = 0;
    }
    if (fixedHeightRadio->isChecked()) {
      wspec.size.fixedH = heightSb->value();
      wspec.size.minH = 0;
    }
  }

  if (forceWidthCb->isChecked()) {
    if (minWidthRadio->isChecked()) {
      wspec.size.minW = widthSb->value();
      wspec.size.fixedW = 0;
    }
    if (fixedWidthRadio->isChecked()) {
      wspec.size.fixedW = widthSb->value();
      wspec.size.minW = 0;
    }
  }

  printf("saved %s\n",(const char *)(widget.toAscii()));

  config->setWidgetSpec(widget,wspec);
}

void ThemeBuilderUI::slot_open()
{
  QString s = QFileDialog::getOpenFileName(NULL,"Load Theme","","*.qsconfig");
  if (!s.isNull()) {
    slot_loadTheme(s);
  }
}

void ThemeBuilderUI::slot_new()
{
  QString s = QFileDialog::getSaveFileName(NULL,"New Theme","","*.qsconfig");
  if (!s.isNull()) {
    QFile::copy(":default.qsconfig",s);
    slot_loadTheme(s);
  }
}
