/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef THEMECONFIG_H
#define THEMECONFIG_H

#include "../themeconfig/specs.h"

class QString;
class QVariant;
class QSettings;
class QStringList;

class ThemeConfig {
  public:
    ThemeConfig();
    ThemeConfig(const QString &theme);
    ~ThemeConfig();

    void load(const QString &theme);

    bool isValid() const { return m_valid; };

    /** Returns the frame spec of the given widget from the theme config file */
    frame_spec_t getFrameSpec(const QString &widgetName) const;
    /** Returns the interior spec of the given widget from the theme config file */
    interior_spec_t getInteriorSpec(const QString &widgetName) const;
    /** Returns the indicator spec of the given widget from the theme config file */
    indicator_spec_t getIndicatorSpec(const QString &widgetName) const;
    /** Returns the label (text+icon) spec of the given widget from the theme config file */
    label_spec_t getTextSpec(const QString &widgetName) const;
    /** Returns the size spec of the given widget from the theme config file */
    size_spec_t getSizeSpec(const QString &widgetName) const;
    /** Returns the widget spec of the given widget from the theme config file */
    widget_spec_t getWidgetSpec(const QString &widgetName) const;

    /** returns the vlue of the given key from the given group from the theme config file */
    QVariant getValue(const QString &group, const QString& key, bool* ok) const;

    void setValue(const QString &group, const QString &key, const QString &v) const;
    void setFrameSpec(const QString &widgetName, const frame_spec_t &fspec) const;
    void setInteriorSpec(const QString &widgetName, const interior_spec_t &ispec) const;
    void setIndicatorSpec(const QString &widgetName, const indicator_spec_t &dspec) const;
    void setTextSpec(const QString &widgetName, const label_spec_t &lspec) const;
    void setSizeSpec(const QString &widgetName, const size_spec_t &sspec) const;
    void setWidgetSpec(const QString &widgetName, const widget_spec_t &wspec) const;

    /** returns the list of widgets contained in the theme config file */
    QStringList getWidgets() const;

  private:
    bool m_valid;
    QSettings *settings;
};

#endif // THEMECONFIG_H
