/***************************************************************************
 *   Copyright (C) 2009 by Saïd LANKRI   *
 *   said.lankri@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SPEC_H
#define SPEC_H

#include <QString>

/** Generic information about a frame */
typedef struct {
  /* Element name */
  QString element;
  /* has frame ? */
  bool hasFrame;
  /* Allow capsule grouping ? */
  bool hasCapsule;
  /* frame size */
  int top,bottom,left,right;
  /* pattern repetition */
  bool rtop,rbottom,rleft,rright;
  /* pattern size */
  int ptop,pbottom,pleft,pright;
  /* widget position in a capsule */
  int capsuleH,capsuleV; // 0 : middle, -1 : left,top, 1 : right,bottom, 2 : left+right,top+bottom
} frame_spec_t;

/** Generic information about a frame interior */
typedef struct {
  /* Element name */
  QString element;
  /* has interior */
  bool hasInterior;
  /* has margins ? */
  bool hasMargin;
  /* interior margins */
  int top,bottom,left,right;
  /* pattern repetition */
  bool rx,ry;
  /* pattern size */
  int px,py;
} interior_spec_t;

/** Generic information about widget indicators */
typedef struct {
  /* Element name */
  QString element;
  /* size */
  int size;
} indicator_spec_t;

/** Generic information about the size of a widget */
typedef struct {
  /* min/fixed width or height. <= 0 : does not apply */
  int minH,fixedH;
  int minW,fixedW;
} size_spec_t;

/** Generic information about text and icons (labels) */
typedef struct {
  /* has shadow */
  bool hasShadow;
  /* shadow shift */
  int xshift,yshift;
  /* shadow color */
  int r,g,b,a;
  /* shadow depth */
  int depth;
  /* has margins ? */
  bool hasMargin;
  /* text margins */
  int top,bottom,left,right;
  /* text-icon spacing */
  int tispace;
} label_spec_t;

/** Generic information about a widget */
typedef struct {
  frame_spec_t frame;
  interior_spec_t interior;
  indicator_spec_t indicator;
  size_spec_t size;
  label_spec_t label;
} widget_spec_t;

#endif
